<?php

/**
 * API2 class for orders(admin)
 *
 * @category   GoDataFeed
 * @package    GoDataFeed_Restapi
 * @author     GoDataFeed Team 
 */
class GoDataFeed_Restapi_Model_Api2_Order_Rest_Admin_V1 extends Mage_Sales_Model_Api2_Order_Rest {

    protected function _retrieve() {
        $order_id = $this->getRequest()->getParam('id');
        if ($order_id) {
            $order = $this->OrderData($order_id, 'get');
            return $order;
        } else {
            $this->error('id'); //'Order id is not exist in this Magento store.';
        }
    }

    protected function error($data) {
        $error = array();
        switch ($data) {
            case "id":
                $message['error'] = array(array('code' => 403, 'message' => 'Please send the Order id.'));
                break;
            case "customer":
                $message['error'] = array(array('code' => 403, 'message' => 'This customer email id is not exist in this Magento store'));
                break;
            case "product":
                $message['error'] = array(array('code' => 403, 'message' => 'This product id is not exist in this Magento store'));
                break;
            case "billing":
                $message['error'] = array(array('code' => 403, 'message' => 'Customer billing address is not set in this Magento store'));
                break;
            case "shipping":
                $message['error'] = array(array('code' => 403, 'message' => 'Customer shipping address is not set in this Magento store'));
                break;
            case "status":
                $message['error'] = array(array('code' => 403, 'message' => 'Order status is not set in order_status'));
                break;
            default:
                $message['error'] = array(array('code' => 403, 'message' => 'Order id is not exist in this Magento store'));
                break;
        }
        $error['messages'] = $message;
        print_r(json_encode($error));
    }

    protected function _create(array $data) {
        // GET ALL Order params Data 
        $order_source_name = $data['order_source_name'];
        $order_source_number = $data['order_source_number'];
        $order_source_created_date = $data['order_source_created_date'];
        $order_payment_trans_id = $data['order_payment_trans_id'];
        $order_payment_method = $data['order_payment_method'];
        $order_shipping_method = $data['order_shipping_method'];
        $order_salestax_total = $data['order_salestax_total'];
        $order_sub_total = $data['order_sub_total'];
        $order_ship_total = $data['order_ship_total'];
        $order_grand_total = $data['order_grand_total'];
        $order_ship_first_name = $data['order_ship_first_name'];
        $order_ship_last_name = $data['order_ship_last_name'];
        $order_ship_company = $data['order_ship_company'];
        $order_ship_street_one = $data['order_ship_street_one'];
        $order_ship_street_two = $data['order_ship_street_two'];
        $order_ship_city = $data['order_ship_city'];
        $order_ship_state = $data['order_ship_state'];
        $order_ship_country = $data['order_ship_country'];
        $order_ship_postal_code = $data['order_ship_postal_code'];
        $order_customer_email = $data['order_customer_email'];
        $order_customer_phone = $data['order_customer_phone'];
        $trackingNum = $data['order_shipping_tracking_num'];
        $order_ship_date = $data['order_ship_date'];
        $order_notes = $data['order_notes'];
        $OrderItems = $data['order_items'];
        $customer = Mage::getModel('customer/customer')->setWebsiteId(1)->loadbyEmail($order_customer_email);
        $customer_id = $customer->getId();
        if (!$customer_id) {
            $this->error('customer');
            die();
        }
        $transaction = Mage::getModel('core/resource_transaction');
        $storeId = $customer->getStoreId();
        $reservedOrderId = Mage::getSingleton('eav/config')->getEntityType('order')->fetchNewIncrementId($storeId);
        $order = Mage::getModel('sales/order')
                ->setIncrementId($reservedOrderId)
                ->setStoreId($storeId)
                ->setQuoteId(0)
                ->setGlobal_currency_code('USD')
                ->setBase_currency_code('USD')
                ->setStore_currency_code('USD')
                ->setOrder_currency_code('USD');
        // set Customer datass
        $order->setCustomer_email($order_customer_email)
                ->setCustomerFirstname($customer->getFirstname())
                ->setCustomerLastname($customer->getLastname())
                ->setCustomerGroupId($customer->getGroupId())
                ->setCustomer_is_guest(0)
                // ->setWebsiteId($customer->getWebsiteId())
                ->setCustomer($customer);
        // set Billing Address
        $billing = $customer->getDefaultBillingAddress();
        $billing_id = $billing->getId();
        if (!$billing_id) {
            $this->error('billing');
            die();
        }
        $billingAddress = Mage::getModel('sales/order_address')
                ->setStoreId($storeId)
                ->setAddressType(Mage_Sales_Model_Quote_Address::TYPE_BILLING)
                ->setCustomerId($customer->getId())
                ->setCustomerAddressId($customer->getDefaultBilling())
                ->setCustomer_address_id($billing->getEntityId())
                ->setPrefix($billing->getPrefix())
                ->setFirstname($billing->getFirstname())
                ->setMiddlename($billing->getMiddlename())
                ->setLastname($billing->getLastname())
                ->setSuffix($billing->getSuffix())
                ->setCompany($billing->getCompany())
                ->setStreet($billing->getStreet())
                ->setCity($billing->getCity())
                ->setCountry_id($billing->getCountryId())
                ->setRegion($billing->getRegion())
                ->setRegion_id($billing->getRegionId())
                ->setPostcode($billing->getPostcode())
                ->setTelephone($billing->getTelephone())
                ->setFax($billing->getFax());
        $order->setBillingAddress($billingAddress);
        $shipping = $customer->getDefaultShippingAddress();
        $shipping_id = $shipping->getId();
        if (!$shipping_id) {
            $this->error('shipping');
            die();
        }
        $shippingAddress = Mage::getModel('sales/order_address')
                ->setStoreId($storeId)
                ->setAddressType(Mage_Sales_Model_Quote_Address::TYPE_SHIPPING)
                ->setCustomerId($customer->getId())
                ->setCustomerAddressId($customer->getDefaultShipping())
                ->setCustomer_address_id($shipping->getEntityId())
                ->setPrefix($shipping->getPrefix())
                ->setFirstname($order_ship_first_name)
                ->setLastname($order_ship_last_name)
                ->setSuffix($shipping->getSuffix())
                ->setCompany($order_ship_company)
                ->setStreet($order_ship_street_one . ' ' . $order_ship_street_two)
                ->setCity($order_ship_city)
                ->setCountry_id($order_ship_country)
                ->setRegion($order_ship_state)
                ->setRegion_id($order_ship_state)
                ->setPostcode($order_ship_postal_code)
                ->setTelephone($order_customer_phone)
                ->setFax($shipping->getFax());
        $order->setShippingAddress($shippingAddress)
                ->setShipping_method('flatrate_flatrate')
                ->setShippingDescription('flatrate');

        $orderPayment = Mage::getModel('sales/order_payment')
                ->setStoreId($storeId)
                ->setCustomerPaymentId(0)
                ->setMethod('purchaseorder')
                ->setPo_number(' - ');
        $order->setPayment($orderPayment);
        // let say, we have 2 products
        $subTotal = 0;
        foreach ($OrderItems as $key => $value) {
            $product_sku = $value['OrderItemSku'];
            $productId = $value['OrderItemId'];
            $_product = $_product = Mage::getModel('catalog/product')->load($productId);
            $product_id = $_product->getId();
            if (!$product_id) {
                $this->error('product');
                die();
            }
            $product_qty = $value['OrderItemQuantity'];
            $product_name = $value['OrderItemName'];
            $product_price = $value['OrderItemUnitPrice'];
            $rowTotal = $product_price * $product_qty;
            $orderItem = Mage::getModel('sales/order_item')
                    ->setStoreId($storeId)
                    ->setQuoteItemId(0)
                    ->setQuoteParentItemId(NULL)
                    ->setProductId($productId)
                    ->setProductType($_product->getTypeId())
                    ->setQtyBackordered(NULL)
                    ->setTotalQtyOrdered($product_qty)
                    ->setQtyOrdered($product_qty)
                    ->setName($product_name)
                    ->setSku($product_sku)
                    ->setPrice($_product->getPrice())
                    ->setBasePrice($product_name)
                    ->setOriginalPrice($product_price)
                    ->setRowTotal($rowTotal)
                    ->setBaseRowTotal($rowTotal);
            $subTotal += $rowTotal;
            $order->addItem($orderItem);
        }
        $order->setSubtotal($subTotal)
                ->setBaseSubtotal($subTotal)
                ->setGrandTotal($subTotal)
                ->setBaseGrandTotal($subTotal);
        $transaction->addObject($order);
        $transaction->addCommitCallback(array($order, 'place'));
        $transaction->addCommitCallback(array($order, 'save'));
        $transaction->save();
        $orderId = Mage::getModel("sales/order")->getCollection()->getLastItem()->getId();
        $orderData = Mage::getModel('sales/order')->loadByIncrementId($orderId);
        $qty = array();
        foreach ($orderData->getAllItems() as $eachOrderItem) {
            $Itemqty = 0;
            $Itemqty = $eachOrderItem->getQtyOrdered() - $eachOrderItem->getQtyShipped() - $eachOrderItem->getQtyRefunded() - $eachOrderItem->getQtyCanceled();
            $qty[$eachOrderItem->getId()] = $Itemqty;
        }
        /* check order shipment is prossiable or not */
        $includeComment = true;
        $comment = "test Shipment";
        $carrier = '';
        if ($orderData->canShip()) {
            /* @var $shipment Mage_Sales_Model_Order_Shipment */
            /* prepare to create shipment */
            $shipment = $orderData->prepareShipment($qty);
            if ($shipment) {
                $carrierTitle = NULL;
                if ($carrier == 'custom') {
                    $carrierTitle = 'courier Service name';
                }
                $data = array();
                $data['carrier_code'] = $carrier;
                $data['title'] = $carrierTitle;
                $data['number'] = $trackingNum;
                $track = Mage::getModel('sales/order_shipment_track')->addData($data);
                $shipment->addTrack($track);
                $shipment->register();
                $shipment->addComment($comment, $email && $includeComment);
                $shipment->getOrder()->setIsInProcess(true);
                try {
                    $transactionSave = Mage::getModel('core/resource_transaction')
                            ->addObject($shipment)
                            ->addObject($shipment->getOrder())
                            ->save();
                } catch (Mage_Core_Exception $e) {
                    var_dump($e);
                }
            }
        }
        $this->OrderData($orderId, 'create');
    }

    protected function _update(array $data) {
        $order_id = $this->getRequest()->getParam('id');
        if ($order_id) {
            $status = $data['order_status'];
            if (!$status) {
                $this->error('status');
                die();
            }
            $order = Mage::getModel('sales/order')->load($order_id);
            $orderId = $order->getId();
            if (!$orderId) {
                $this->error('order');
                die();
            }
            switch ($status) {
                case "processing":
                    $order->setState(Mage_Sales_Model_Order::STATE_PROCESSING, true);
                    break;
                case "closed":
                    $order->setState(Mage_Sales_Model_Order::STATE_CLOSED, true);
                    break;
                case "holded":
                    $order->setState(Mage_Sales_Model_Order::STATE_HOLDED, true);
                    break;
                case "canceled":
                    $order->setState(Mage_Sales_Model_Order::STATE_CANCELED, true);
                    break;
            }
            $order->save();
            $this->OrderData($order_id, 'update');
        } else {
            $this->error('id'); //'Order id is not exist in this Magento store.'; 
        }
    }

    protected function _delete(array $data) {
        print_r($data);
    }

    protected function _retrieveCollection() {
        $order_id = $this->getRequest()->getParam('id');
        if ($order_id) {
            $order = $this->OrderData($order_id, 'get');
            return $order;
        } else {
            $this->error('id');
        }
    }

    // This function is used to fetch order details from by Order Id.
    public function orderData($order_id, $method_name) {
        $order = array();
        $orderData = Mage::getModel('sales/order')->load($order_id);
        $orderId = $orderData->getId();
        if (!$orderId) {
            $this->error('order');
            die();
        }
        $order['order_id'] = $order_id;
        $order['customer_email'] = $orderData->getCustomerEmail();
        $order['order_grand_total'] = $orderData->getGrandTotal();
        $order['order_sub_total'] = $orderData->getSubtotal();
        $order['order_shipping_method'] = $orderData->getShippingMethod();
        $order['order_ship_total'] = $orderData->getShippingAmount();
        $order['order_salestax_total'] = $orderData->getTaxAmount();
        $order['order_status'] = $orderData->getStatus();
        $billingAddress = $orderData->getBillingAddress();
        $shippingAddress = $orderData->getShippingAddress();
        $order['billing_address'] = array($billingAddress->getData());
        $order['shipping_address'] = array($shippingAddress->getData());
        $shipmentCollection = $orderData->getShipmentsCollection();
        $tracknums = array();
        foreach ($shipmentCollection as $shipment) {
            // This will give me the shipment IncrementId, but not the actual tracking information.
            $order['order_ship_date'] = $shipment->getCreatedAt();
            foreach ($shipment->getAllTracks() as $tracknum) {
                $tracknums[] = $tracknum->getNumber();
            }
        }
        $order['order_shipping_tracking_num'] = implode('|', $tracknums);
        $order_items = array();
        $ordered_items = $orderData->getAllItems();
        foreach ($ordered_items as $key => $item) {
            $order_item = array();
            $order_item['OrderItemId'] = $item->getItemId();
            $order_item['OrderItemSku'] = $item->getSku();
            $order_item['OrderItemQuantity'] = $item->getQtyOrdered();
            $order_item['OrderItemName'] = $item->getName();
            $order_item['OrderItemUnitPrice'] = $item->getPrice();
            $order_item['OrderItemOriginalPrice'] = $item->getOriginalPrice();
            $order_item['OrderItemName'] = $item->getName();
            $order_item['OrderItemWeight'] = $item->getWeight();
            $order_items[$key] = $order_item;
        }
        $order['order_items'] = array($order_items);
        if ($method_name == 'update' || $method_name == 'create') {
            print_r(json_encode($order));
        } else {
            return $order;
        }
    }

}
